@@ -35,7 +35,7 @@
 #include "db.h"				/* db data header		*/
 #include "util.h"                       /* utility function header      */
 
-/* databse globals , this needs to be eliminated !! */	
+/* database globals , this needs to be eliminated !! */	
 	
 PGconn *db = NULL;		        /* db ptr		        */
 char dbname[DB_NAME_SZ];		/* db name 	         	*/
@@ -47,6 +47,33 @@ char spool_file[MAX_PATH_LEN];          /* spool file                   */
 char printer[PRINT_NAME_SZ];            /* printer                      */
 int connected = 0;                      /* connect flag                 */
 int user_prompt = 0;                    /* user & pwd prompt flag       */
+int qy_cancelled = 0;                   /* query cancel flag            */
+int qy_running = 0;                     /* query running flag           */
+
+/************************************************************************/
+/* Checks if user has selected to cancel the currently running query    */
+/************************************************************************/
+static int check_cancelled(void)
+{
+  Display *dpy = XtDisplay(AppWidgetsPtr->shell);
+  Window win = XtWindow(AppWidgetsPtr->btn_exe);
+  XEvent event;
+
+  /* Update the display */
+  XFlush(dpy);
+  XmUpdateDisplay(AppWidgetsPtr->shell);
+
+  /* Check if the cancel button was pressed */
+  while(XCheckMaskEvent(dpy,
+			ButtonPressMask | ButtonReleaseMask | KeyPressMask,
+			&event)){
+    if(event.xany.window == win)
+      XtDispatchEvent(&event);
+    else
+      XBell(dpy, 50);
+  }
+  return qy_cancelled;
+}
 
 /************************************************************************/
 /* Function: DB_Connect                                                 */
@@ -126,6 +153,7 @@ int DB_SubmitQy(void)
   int buf_pos;
   int qy_pos;
   int qy_len;
+  int in_quote = 0;
   int selected = 0;
   char db_msg[ERR_MSG_SIZE];
   char qy_buffer[QY_BUFFER_SIZE];
@@ -155,18 +183,26 @@ int DB_SubmitQy(void)
  
   qy_pos = 0;
 
-  XUTIL_BeginWait(AppWidgetsPtr->mainwindow);
-
   while(qy_pos < qy_len){
     buf_pos = 0;
 
-    while((qy_ptr[qy_pos] != ';') && (qy_pos < qy_len)){
+    MQUEL_SetToolbar(0);
+
+    while(qy_pos < qy_len){
+      if((qy_ptr[qy_pos] == '\'') && !in_quote)
+	in_quote = 1;
+      else if((qy_ptr[qy_pos] == '\'') && in_quote)
+	in_quote = 0;
+
+      if(!in_quote && (qy_ptr[qy_pos] == ';'))
+	 break;
+
       qy_buffer[buf_pos++] = qy_ptr[qy_pos++];
       if(strlen(qy_buffer) > (QY_BUFFER_SIZE - 2)){
-	sprintf(db_msg,"Query length exceeds max. query size, truncated at %d\n", 
+    	sprintf(db_msg,"Query length exceeds max. query size, truncated at %d\n", 
 		QY_BUFFER_SIZE);
-	DB_WriteToResult(db_msg, 0, NULL);
-	break;
+    	DB_WriteToResult(db_msg, 0, NULL);
+    	break;
       }
     }
 
@@ -174,15 +210,33 @@ int DB_SubmitQy(void)
     qy_buffer[buf_pos++] = ';';
     qy_buffer[buf_pos] = '\0';
 
-    results = PQexec(db, (char*)doubleTrim(qy_buffer));
-    if (results == NULL) {
-      XUTIL_EndWait(AppWidgetsPtr->mainwindow);
+    // Submit the query
+    if (!PQsendQuery(db, (char*)doubleTrim(qy_buffer))){
       DB_WriteToResult(PQerrorMessage(db), 0, NULL);
+      DB_WriteToResult("\n", 0, NULL);
       TextDMakeInsertPosVisible(((TextWidget)AppWidgetsPtr->resultwindow)->text.textD);
       XtFree((char*)qy_ptr);
+      MQUEL_SetToolbar(1);
       return 0;
     }
-    
+
+    qy_running = 1;
+    qy_cancelled = 0;
+
+    while(1) {
+      PQconsumeInput(db);
+
+      if(!PQisBusy(db))
+	break;
+      else
+      	if(check_cancelled()){
+	  PQrequestCancel(db);
+	  break;
+	}
+    }
+
+    results = PQgetResult(db);
+
     switch (PQresultStatus(results)) {
     case PGRES_TUPLES_OK:
       DB_PrintTuples(qy_buffer,
@@ -213,10 +267,12 @@ int DB_SubmitQy(void)
     case PGRES_NONFATAL_ERROR:
     case PGRES_FATAL_ERROR:
     case PGRES_BAD_RESPONSE:
+      qy_cancelled = 1;
       DB_WriteToResult(PQerrorMessage(db), 0, NULL);
-      break;
+      /* reset the connection -  ???*/
+      DB_Connect();
     } 
-    
+   
     /* check for asynchronous returns */
     notify = PQnotifies(db);
     if (notify) {
@@ -225,12 +281,31 @@ int DB_SubmitQy(void)
       DB_WriteToResult(db_msg, 0, NULL);
       free(notify);
     }
+    if(qy_cancelled){
+      qy_cancelled = 0;
+      break;
+    }
   }
   XtFree((char*)qy_ptr);
-  XUTIL_EndWait(AppWidgetsPtr->mainwindow);
+  MQUEL_SetToolbar(1);
+  qy_running = 0;
   return 1;
 }
 
+/************************************************************************/
+/* Function: DB_CancelQuery                                             */
+/* Purpose : cancels a executing query on the current connection        */
+/* Params  : none                                                       */
+/* Returns : nothing                                                    */
+/* Notes   :                                                            */
+/************************************************************************/
+
+void DB_CancelQy(void)
+{
+  if(qy_running)
+    qy_cancelled = 1;
+}
+
 /************************************************************************/
 /* Function: DB_OpenFile                                                */
 /* Purpose : opens a file into the SQL window                           */
@@ -379,7 +454,7 @@ static void do_field(PQprintOpt *po, PGresult *res,
 {
   char *pval, *p, *o;
   int plen;
-  bool skipit;
+  int skipit;
   int Spool;
   char dataString[DATA_SIZE];
   
@@ -394,15 +469,15 @@ static void do_field(PQprintOpt *po, PGresult *res,
   if (plen < 1 || !pval || !*pval)
     {
       if (po->align || po->expanded)
-	skipit = true;
+	skipit = 1;
       else
 	{
-	  skipit = false;
+	  skipit = 0;
 	  goto efield;
 	}
     }
   else
-    skipit = false;
+    skipit = 0;
   
   if (!skipit)
     {
@@ -501,7 +576,7 @@ static char * do_header(FILE *fout, PQprintOpt *po, const int nFields, int field
 	tot += fieldMax[n] + fs_len + (po->standard ? 2 : 0);
       if (po->standard)
 	tot += fs_len * 2 + 2;
-      border = malloc(tot + 1);
+      border = (char*)malloc(tot + 1);
       if (!border)
 	{
 	  perror("malloc");
@@ -526,7 +601,7 @@ static char * do_header(FILE *fout, PQprintOpt *po, const int nFields, int field
 	      
 	      while (*fs++)
 		*p++ = ' ';
-	    }
+	    } 
 	}
       *p = '\0';
       if (po->standard){
@@ -534,8 +609,10 @@ static char * do_header(FILE *fout, PQprintOpt *po, const int nFields, int field
 	DB_WriteToResult(dataString, Spool, fout);
       }
     }
-  if (po->standard)
-    fputs(po->fieldSep, fout);
+  if (po->standard){
+    sprintf(dataString, "%s", po->fieldSep);
+    DB_WriteToResult(dataString, Spool, fout);
+  }
   for (j = 0; j < nFields; j++)
     {
       char *s = PQfname(res, j);
@@ -637,7 +714,7 @@ void DB_FormatResults(FILE *fout, PGresult *res, PQprintOpt *po)
   
   nFields = PQnfields(res);
 
-  if (nFields > 0)
+  if(nFields > 0)
     {
       int i, j;
       int nTups;
@@ -673,11 +750,12 @@ void DB_FormatResults(FILE *fout, PGresult *res, PQprintOpt *po)
 	  perror("calloc");
 	  exit(1);
 	}
-     
+ 
       for (j = 0; j < nFields; j++)
 	{
 	  int len;
 	  char *s = PQfname(res, j);
+	 
 	    
 	  fieldNames[j] = s;
 	  len = s ? strlen(s) : 0;
@@ -721,8 +799,7 @@ void DB_FormatResults(FILE *fout, PGresult *res, PQprintOpt *po)
 	      for (j = 0; j < nFields; j++)
 		{
 		  char	   *s = fieldNames[j];
- 
-		  sprintf(dataString, s);
+		  strcat(dataString, s);
 		  len += strlen(s) + fs_len;
 		  if ((j + 1) < nFields)
 		    strcat(dataString, po->fieldSep);
@@ -735,24 +812,22 @@ void DB_FormatResults(FILE *fout, PGresult *res, PQprintOpt *po)
 	}
       if (po->expanded && po->html3)
 	{
-	  if (po->caption)
-	    sprintf(dataString, "<center><h2>%s</h2></center>\n", po->caption);
-	  else
-	    sprintf(dataString,
-		    "<center><h2>"
-		    "Query retrieved %d rows * %d fields"
-		    "</h2></center>\n",
-		    nTups, nFields);
+	  sprintf(dataString,
+		  "<center><h2>"
+		  "Query retrieved %d rows * %d fields"
+		  "</h2></center>\n",
+		  nTups, nFields);
 	  DB_WriteToResult(dataString, Spool, fout);
 	}
       for (i = 0; i < nTups; i++)
 	{
+	  if(check_cancelled())
+	    break;
 	  if (po->expanded)
 	    {
 	      if (po->html3)
 		sprintf(dataString,
-			"<table %s><caption align=high>%d</caption>\n",
-			po->tableOpt ? po->tableOpt : "", i);
+			"<table><caption align=high>%d</caption>\n", i);
 	      else
 		sprintf(dataString, "-- RECORD %d --\n", i);
 	      DB_WriteToResult(dataString, Spool, fout);
@@ -770,21 +845,15 @@ void DB_FormatResults(FILE *fout, PGresult *res, PQprintOpt *po)
 	    {
 	      if (po->header) 
 		{
-		  if (po->caption)
-		    sprintf(dataString,
-			    "<table %s><caption align=high>%s</caption>\n",
-			    po->tableOpt ? po->tableOpt : "",
-			    po->caption);
-		  else
-		    sprintf(dataString,
-			    "<table %s><caption align=high>"
-			    "Retrieved %d rows * %d fields"
-			    "</caption>\n",
-			    po->tableOpt ? po->tableOpt : "", nTups, nFields);
+		  sprintf(dataString,
+		  	  "<table><caption align=high>"
+		  	  "Retrieved %d rows * %d fields"
+		  	  "</caption>\n",
+		  	  nTups, nFields);
 		  DB_WriteToResult(dataString, Spool, fout);
 		}
 	      else {
-		sprintf(dataString, "<table %s>", po->tableOpt ? po->tableOpt : "");
+		sprintf(dataString, "<table>");
 		DB_WriteToResult(dataString, Spool, fout);
 	      }
 	    }
@@ -792,6 +861,8 @@ void DB_FormatResults(FILE *fout, PGresult *res, PQprintOpt *po)
 	    border = do_header(fout, po, nFields, fieldMax, fieldNames,
 	    		       fieldNotNum, fs_len, res);
 	  for (i = 0; i < nTups; i++){
+	    if(check_cancelled())
+	      break;
 	    output_row(fout, po, nFields, fields,
 	    	       fieldNotNum, fieldMax, border, i);
 	    if(TextGetCursorPos(AppWidgetsPtr->resultwindow) > RSLT_BUFFER_SIZE)
@@ -893,7 +964,7 @@ void DB_WriteToResult(char *data, int spool, FILE *file)
 {
   textBuffer *buffer;
   int ins_pos;
-
+  
   if(spool)
     fprintf(file, data);
   else{
@@ -1214,8 +1285,7 @@ void DB_HandleCopyOut(PGresult *res)
     int ret;
     
     while (!copydone) {
-	ret = PQgetline(res->conn, copybuf, COPYBUFSIZ);
-	
+	ret = PQgetline(db, copybuf, COPYBUFSIZ);
 	if (copybuf[0] == '.' && copybuf[1] =='\0') {
 	    copydone = 0;	/* don't print this... */
 	} else {
@@ -1232,7 +1302,7 @@ void DB_HandleCopyOut(PGresult *res)
 	}
     }
     fflush(stdout);
-    PQendcopy(res->conn);
+    PQendcopy(db);
 }
 
 /************************************************************************/
@@ -1271,12 +1341,12 @@ void DB_HandleCopyIn(PGresult *res)
 	    }
 	    if (c == EOF) {
 		/* reading from stdin, but from a file */
-		PQputline(res->conn, ".");
+		PQputline(db, ".");
 		copydone = 0;
 		break;
 	    }
 	    *s = '\0';
-	    PQputline(res->conn, copybuf);
+	    PQputline(db, copybuf);
 	    if (firstload) {
 		if (!strcmp(copybuf, ".")) {
 		    copydone = 0;
@@ -1284,9 +1354,9 @@ void DB_HandleCopyIn(PGresult *res)
 		firstload = 1;
 	    }
 	}
-	PQputline(res->conn, "\n");
+	PQputline(db, "\n");
     }
-    PQendcopy(res->conn);
+    PQendcopy(db);
 }
 
 /************************************************************************/
