@@ -1,9 +1,9 @@
 /************************************************************************/
-/* Module :  menu.c					                */
+/* Module : menu.c					                */
 /* Purpose: menu module				                        */
-/* By     :      Keith R. Davis				                */
-/* Date   :    9/9/95					                */
-/* Notes  :   Copyright(c) 1996 White River Software	                */
+/* By     : Keith R. Davis				                */
+/* Date   : 9/9/95					                */
+/* Notes  : Copyright(c) 1996-99 Mutiny Bay Software	                */
 /************************************************************************/
 
 #include <Xm/Xm.h>			/* motif lib header		*/
@@ -20,6 +20,7 @@
 #include "mquel.h"                      /* mquel module header          */
 #include "menu.h"			/* menu module header	        */
 #include "callback.h"                   /* callback header              */
+#include "search.h"                     /* search header                */
 
 /************************************************************************/
 /* Function: CreateMenuBar                                              */
@@ -38,6 +39,7 @@ Widget CreateMenuBar(Widget parent)
 
     CreateFilePane(menu);
     CreateEditPane(menu);
+    CreateSearchPane(menu);
     CreateQueryPane(menu);
     CreateOptionsPane(menu);
     CreateBufferPane(menu);
@@ -210,6 +212,56 @@ void CreateEditPane(Widget parent)
   XtAddCallback(button4, XmNactivateCallback, ClearResultCallback, NULL);
 }
 
+/************************************************************************/
+/* Function: CreateSearchPane                                           */
+/* Purpose : creates the applications search menu                       */
+/* Params  : parent : menu's parent widget                              */
+/* Returns : nothing                                                    */
+/* Notes   :                                                            */
+/************************************************************************/
+
+void CreateSearchPane(Widget parent)
+{
+  Widget	cascade,	/* cascade menu widget	        */
+                submenu,	/* pull down menu widget	*/
+		button1,	/* find menu button	        */
+                button2,        /* find again menu button	*/
+                button3;        /* replace menu button	        */
+
+  /* create the pulldown part of the query menu */
+  submenu = XmCreatePulldownMenu(parent, "searchSubmenu",
+				 NULL, 0);
+
+  /* add help callback for the edit menu */
+  XtAddCallback(submenu, XmNhelpCallback,
+		  get_help, "mpsql_menu_search.html");
+
+  /* create  the 'SEARCH' menu */
+  cascade = XtVaCreateManagedWidget("search",
+				    xmCascadeButtonWidgetClass,
+				    parent,
+				    XmNsubMenuId, submenu,
+				    NULL);
+
+  /*add the selections to the 'SEARCH' menu */
+  button1 = XtCreateManagedWidget("find",
+				  xmPushButtonWidgetClass,
+				  submenu, NULL, 0);
+  
+  button2 = XtCreateManagedWidget("findagain",
+				  xmPushButtonWidgetClass,
+				  submenu, NULL, 0);
+
+  button3 = XtCreateManagedWidget("replace",
+				  xmPushButtonWidgetClass,
+				  submenu, NULL, 0);
+  
+  /* install the callback functions */
+  XtAddCallback(button1, XmNactivateCallback, SEARCH_FindDlg, NULL);
+  XtAddCallback(button2, XmNactivateCallback, SEARCH_FindAgain, NULL);
+  XtAddCallback(button3, XmNactivateCallback, SEARCH_ReplaceDlg, NULL);
+}
+
 /************************************************************************/
 /* Function: CreateQueryPane                                            */
 /* Purpose : creates the applications query  menu                       */
@@ -267,6 +319,7 @@ void CreateOptionsPane(Widget parent)
                 sepselect,      /* delimtier menu button        */
                 outselect,      /* output menu button           */
                 syntaxmenu,     /* syntax highlight menu        */
+                tabmenu,        /* tab settings menu            */
                 alignmenu,      /* output alignment menu        */
                 outtypemenu,    /* output type menu             */
                 sepmenu,        /* delimiter menu               */
@@ -319,22 +372,27 @@ void CreateOptionsPane(Widget parent)
 						   XmNfillOnSelect, TRUE,
 						   NULL);
 
-     AppWidgetsPtr->plain = XtVaCreateManagedWidget("plain", xmToggleButtonWidgetClass,
+    AppWidgetsPtr->plain = XtVaCreateManagedWidget("plain", xmToggleButtonWidgetClass,
 						       syntaxselect, 
 						       XmNvisibleWhenOff, TRUE,
 						       XmNindicatorSize, 15,
 						       XmNfillOnSelect, TRUE,
 						       NULL);
 
+    tabmenu = XtVaCreateManagedWidget("tabmenu",
+				      xmCascadeButtonWidgetClass,
+				      submenu,
+				      NULL);
+
     sep3 = XtCreateManagedWidget("sep3", 
-				 xmSeparatorWidgetClass,
-				 submenu, NULL, 0);
+				  xmSeparatorWidgetClass,
+				  submenu, NULL, 0);
 
     sepselect = XmCreatePulldownMenu(submenu, "sepselect",
-				    NULL, 0);
-
+				      NULL, 0);
+     
     XtVaSetValues(sepselect, XmNradioBehavior, TRUE, NULL);
-
+     
     sepmenu = XtVaCreateManagedWidget("sepmenu",
 				    xmCascadeButtonWidgetClass,
 				    submenu,
@@ -479,6 +537,7 @@ void CreateOptionsPane(Widget parent)
     /* install the callback functions */
     XtAddCallback(AppWidgetsPtr->spoolfile, XmNactivateCallback, SpoolCallback, NULL);
     XtAddCallback(AppWidgetsPtr->highlight, XmNvalueChangedCallback, HighlightToggleCallback, NULL);
+    XtAddCallback(tabmenu, XmNactivateCallback, TabCallback, NULL);
     XtAddCallback(button3, XmNactivateCallback, SaveOptCallback, NULL);
 }
 
@@ -576,8 +635,8 @@ void CreateHelpPane(Widget parent)
 
     /* install the callback functions */
     XtAddCallback(button1, XmNactivateCallback, get_help, "mpsql.html");
-    XtAddCallback(button3, XmNactivateCallback, get_help, "pgsql.html");
-    XtAddCallback(button4, XmNactivateCallback, get_help, "sql.html");
+    XtAddCallback(button3, XmNactivateCallback, get_help, "user.html");
+    XtAddCallback(button4, XmNactivateCallback, get_help, "sql-commands.html");
     XtAddCallback(button2, XmNactivateCallback, AboutCallback, NULL);
 }
 
