@@ -64,15 +64,25 @@ WindowInfo WinData;
 WindowInfo *WinDataPtr = &WinData;
 WindowInfo *WindowList = &WinData;
 
-/* tooltip actions and translations */
-#ifdef TOOLTIP
+/* actions and translations */
 XtActionsRec actions[] = {
+#ifdef TOOLTIP
   { "PopupTooltip",        (XtActionProc) PopupTooltip   },
   { "PopdownTooltip",      (XtActionProc) PopdownTooltip },
-};
-char transTable[] = "#<EnterWindow>:PopupTooltip()\n<LeaveWindow>:PopdownTooltip()";
-XtTranslations toolTrans;
 #endif
+  { "DB_CancelQy",         (XtActionProc) DB_CancelQy },
+};
+
+#ifdef TOOLTIP
+char transTable[] = "#<EnterWindow>:PopupTooltip()\n<LeaveWindow>:PopdownTooltip()\nCtrl<Key>Z:DB_CancelQy()\n";
+#else
+char transTable[] = "Ctrl<Key>Z:DB_CancelQy()";
+#endif
+
+XtTranslations appTrans;
+
+/* editor globals */
+int tabs;
 
 /* Main: entry point */
 void main (int argc, char **argv)
@@ -286,16 +296,17 @@ void main (int argc, char **argv)
     XtAddCallback(AppWidgets.btn_connect, XmNhelpCallback,
 		  get_help, "mpsql_toolbar.html");
 
-#ifdef TOOLTIP
     XtAppAddActions(app, actions, XtNumber(actions));
-    toolTrans = XtParseTranslationTable(transTable);
-    XtOverrideTranslations(AppWidgets.btn_new, toolTrans);
-    XtOverrideTranslations(AppWidgets.btn_open, toolTrans);
-    XtOverrideTranslations(AppWidgets.btn_save, toolTrans);
-    XtOverrideTranslations(AppWidgets.btn_print, toolTrans);
-    XtOverrideTranslations(AppWidgets.btn_exe, toolTrans);
-    XtOverrideTranslations(AppWidgets.btn_spool, toolTrans);
-    XtOverrideTranslations(AppWidgets.btn_connect, toolTrans);
+    appTrans = XtParseTranslationTable(transTable);
+    XtAugmentTranslations(AppWidgets.btn_exe, appTrans);
+#ifdef TOOLTIP
+    XtOverrideTranslations(AppWidgets.btn_new, appTrans);
+    XtOverrideTranslations(AppWidgets.btn_open, appTrans);
+    XtOverrideTranslations(AppWidgets.btn_save, appTrans);
+    XtOverrideTranslations(AppWidgets.btn_print, appTrans);
+    XtOverrideTranslations(AppWidgets.btn_exe, appTrans);
+    XtOverrideTranslations(AppWidgets.btn_spool, appTrans);
+    XtOverrideTranslations(AppWidgets.btn_connect, appTrans);
 #endif
 
 
@@ -307,28 +318,12 @@ void main (int argc, char **argv)
 					      NULL);
 
     /* setup the SQL window */
-    /*AppWidgets.sqlwindow = XmCreateScrolledText(AppWidgets.pane, "sqlwindow", NULL, 0);*/
     AppWidgets.sqlwindow = MQUEL_CreateSqlTextArea(AppWidgets.pane);
     XtManageChild(AppWidgets.sqlwindow);
-    /*XtVaSetValues(AppWidgets.sqlwindow,
-		  XmNeditMode, XmMULTI_LINE_EDIT,
-		  NULL);*/
-
-    /* add callback for the SQL window */
-    /*XtAddCallback(AppWidgets.sqlwindow, XmNmodifyVerifyCallback,
-		  TextModifiedCallback, NULL);*/
-
-    /* add help callback for the SQL window */
-    /*XtAddCallback(AppWidgets.sqlwindow, XmNhelpCallback,
-		  get_help, "mpsql_edit.html");*/
-   
+    
     /* setup the result window */
     AppWidgets.resultwindow = MQUEL_CreateResultTextArea(AppWidgets.pane);
    
-    /* add help callback for the results window */
-    /*XtAddCallback(AppWidgets.resultwindow, XmNhelpCallback,
-		  get_help, "mpsql_output.html");*/
-
     /* create the menu */
     AppWidgets.menu = CreateMenuBar(AppWidgets.mainwindow);
 
@@ -469,7 +464,7 @@ Widget MQUEL_CreateSqlTextArea(Widget parent)
 				   textNvScrollBar, vScrollBar,
 				   textNautoWrap, 0,
 				   textNcontinuousWrap, 0,
-				   textNemulateTabs, 4,
+				   textNemulateTabs, tabs,
 				   NULL);
 
     XtVaSetValues(sw, XmNworkWindow, text, XmNhorizontalScrollBar, hScrollBar,
@@ -522,6 +517,35 @@ Widget MQUEL_CreateResultTextArea(Widget parent)
     return text;
 }
 
+/************************************************************************/
+/* Function: MQUEL_SetToolbar                                           */
+/* Purpose : set the application toolbar state                          */
+/* Params  : int  1=enabled, 0=disabled                                 */
+/* Returns : nothing                                                    */
+/* Notes   :                                                            */
+/************************************************************************/
+
+void MQUEL_SetToolbar(int state){
+  if(!state){
+    XtSetSensitive(AppWidgetsPtr->btn_new, False);
+    XtSetSensitive(AppWidgetsPtr->btn_open, False);
+    XtSetSensitive(AppWidgetsPtr->btn_save, False);
+    XtSetSensitive(AppWidgetsPtr->btn_print, False);
+    XtSetSensitive(AppWidgetsPtr->btn_spool, False);
+    XtSetSensitive(AppWidgetsPtr->btn_connect, False);
+    InstallLabeledPixmap(AppWidgetsPtr->btn_exe, cancel_xpm, SHOWLBL);
+  }
+  else {
+    XtSetSensitive(AppWidgetsPtr->btn_new, True);
+    XtSetSensitive(AppWidgetsPtr->btn_open, True);
+    XtSetSensitive(AppWidgetsPtr->btn_save, True);
+    XtSetSensitive(AppWidgetsPtr->btn_print, True);
+    XtSetSensitive(AppWidgetsPtr->btn_spool, True);
+    XtSetSensitive(AppWidgetsPtr->btn_connect, True);
+    InstallLabeledPixmap(AppWidgetsPtr->btn_exe, execute_xpm, SHOWLBL);
+  }
+}
+
 /************************************************************************/
 /* Function: MQUEL_GetOptions                                           */
 /* Purpose : gets app options from setup file                           */
@@ -540,7 +564,7 @@ int MQUEL_GetOptions(void)
   char key_buffer[30];
   char val_buffer[255];
 
-  strcat(strcpy(home, getenv("HOME")), "/.mpsql_cfg");
+  strcat(strcpy(home, getenv("HOME")), "/.mpsqlrc");
 
   if((optf = fopen(home, "r")) != NULL){
     
@@ -564,7 +588,9 @@ int MQUEL_GetOptions(void)
 	if(strcmp("PRINTER", key_buffer) == 0)
 	  strcpy(printer, val_buffer);
 	if(strcmp("SPOOL_FILE", key_buffer) == 0)
-	  strcpy(spool_file, val_buffer); 
+	  strcpy(spool_file, val_buffer);
+	if(strcmp("TABS", key_buffer) == 0)
+	  tabs = atoi(val_buffer);
 	if(strcmp("SYNTAX_HIGHLIGHT", key_buffer) == 0)
 	  SetHighlight(atoi(val_buffer));
 	if(strcmp("COLUMN_SEP", key_buffer) == 0)
@@ -596,6 +622,7 @@ int MQUEL_GetOptions(void)
    strcpy(printer, "lpr");
    strcpy(spool_file, "/tmp/spool.txt");
    SetHighlight(1);
+   tabs = 3;	
    SetColSeparator(SPACE);
    SetAlignment(0);
    SetType(0);
@@ -617,7 +644,7 @@ int MQUEL_SaveOptions(void)
   char home[MAX_PATH_LEN];  
   char db_msg[ERR_MSG_SIZE];   
 
-  strcat(strcpy(home, getenv("HOME")), "/.mpsql_cfg");
+  strcat(strcpy(home, getenv("HOME")), "/.mpsqlrc");
 
   if((optf = fopen(home, "w+")) == NULL){
     sprintf(db_msg, "Error opening options file:\n%s", home);
@@ -637,6 +664,7 @@ int MQUEL_SaveOptions(void)
   fprintf(optf, "PRINTER\t\t\t%s\n", printer, optf);
   fprintf(optf, "SPOOL_FILE\t\t%s\n", spool_file, optf);
   fprintf(optf, "SYNTAX_HIGHLIGHT\t%d\n", GetHighlight());
+  fprintf(optf, "TABS\t\t\t%d\n", tabs);
   fprintf(optf, "COLUMN_SEP\t\t%d\n", GetColSeparatorId());
   fprintf(optf, "OUTPUT_ALIGN\t\t%d\n", GetAlignment());
   fprintf(optf, "OUTPUT_TYPE\t\t%d\n", GetType());
