@@ -15,6 +15,7 @@
 #include "../pixmaps/mpsql/color/save.xpm"
 #include "../pixmaps/mpsql/color/print.xpm"
 #include "../pixmaps/mpsql/color/execute.xpm"
+#include "../pixmaps/mpsql/color/cancel.xpm"
 #include "../pixmaps/mpsql/color/spool.xpm"
 #include "../pixmaps/mpsql/color/connect.xpm"
 #include "../pixmaps/mpsql/color/about.xpm"
@@ -24,7 +25,8 @@
 #include "../pixmaps/mpsql/bw/open.xpm"
 #include "../pixmaps/mpsql/bw/save.xpm"
 #include "../pixmaps/mpsql/bw/print.xpm"
-#include "../pixmaps/mpsql/bw/execute.xpm" 
+#include "../pixmaps/mpsql/bw/execute.xpm"
+#include "../pixmaps/mpsql/bw/cancel.xpm"
 #include "../pixmaps/mpsql/bw/spool.xpm"
 #include "../pixmaps/mpsql/bw/connect.xpm"
 #include "../pixmaps/mpsql/bw/about.xpm"
