@@ -25,5 +25,8 @@ void reverse(char *s);
 
 /* converts int value to character */
 char* itoa(int n, char *s);
+
+/* removes white space from a string */
+void removeWhiteSpace(char *string);
   
 #endif
